@@ -11,7 +11,7 @@ from rest_framework.settings import api_settings
 from rest_framework_csv.renderers import CSVRenderer
 
 from .models import ContributorRole, Contribution
-from .serializers import ContributorRoleSerializer, ContributionReportingSerializer
+from .serializers import ContributorRoleSerializer, ContributionReportingSerializer, ContentReportingSerializer
 from bulbs.content.models import Content
 
 # User = get_user_model()
@@ -22,6 +22,31 @@ class ContributorRoleViewSet(viewsets.ModelViewSet):
     serializer_class = ContributorRoleSerializer
 
 
+class ContentReportingViewSet(viewsets.GenericViewSet, mixins.ListModelMixin):
+
+    renderer_classes = (CSVRenderer, ) + tuple(api_settings.DEFAULT_RENDERER_CLASSES)
+    serializer_class = ContentReportingSerializer
+
+    def get_queryset(self):
+        now = timezone.now()
+
+        start_date = datetime.datetime(
+            year=now.year,
+            month=now.month,
+            day=1,
+            tzinfo=now.tzinfo)
+        if "start" in self.request.GET:
+            start_date = dateparse.parse_date(self.request.GET["start"])
+
+        end_date = now
+        if "end" in self.request.GET:
+            end_date = dateparse.parse_date(self.request.GET["end"])
+
+        content = Content.objects.filter(published__range=(start_date, end_date))
+
+        return content
+
+
 class ReportingViewSet(viewsets.GenericViewSet, mixins.ListModelMixin):
 
     renderer_classes = (CSVRenderer, ) + tuple(api_settings.DEFAULT_RENDERER_CLASSES)
@@ -57,3 +82,4 @@ class ReportingViewSet(viewsets.GenericViewSet, mixins.ListModelMixin):
 api_v1_router = routers.DefaultRouter()
 api_v1_router.register(r"role", ContributorRoleViewSet, base_name="contributorrole")
 api_v1_router.register(r"reporting", ReportingViewSet, base_name="contributionreporting")
+api_v1_router.register(r"contentreporting", ContentReportingViewSet, base_name="contentreporting")
